// -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/***
  BEGIN LICENSE

  Copyright (C) 2013 Julien Spautz <spautz.julien@gmail.com>
  This program is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License version 3, as published
  by the Free Software Foundation.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranties of
  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
  PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program.  If not, see <http://www.gnu.org/licenses/>

  END LICENSE
***/

public const string NAME = _("Folder Manager");
public const string DESCRIPTION = _("Basic folder manager with file browsing");

namespace Scratch.Plugins {
    public class FolderManagerPlugin : Peas.ExtensionBase, Peas.Activatable {
        public Scratch.Services.Interface plugins;
        Gtk.Button button;
        FolderManager.FileView view;

        public Object object { owned get; construct; }

        public FolderManagerPlugin () {
            message ("Starting Folder Manager Plugin");
        }

        public void activate () {
            plugins = (Scratch.Services.Interface) object;
            plugins.hook_notebook_sidebar.connect (on_hook_sidebar);
            plugins.hook_toolbar.connect (on_hook_toolbar);
        }

        public void deactivate () {
            if (view != null) {
                view.destroy();
                view = null;
            }

            if (button != null) {
                button.destroy();
                button = null;
            }
        }

        public void update_state () {
            
        }

        void on_hook_sidebar (Gtk.Notebook notebook) {
            if (view != null) {
                return;
            }

            // File View
            view = new FolderManager.FileView ();
            view.select.connect ((file) => plugins.open_file (file));
            view.welcome_visible.connect ((visible) => {
                if (visible) {
                    view.parent.remove (view);
                } else if (view.parent == null) {
                    view.show_all ();
                    var icon = new Gtk.Image.from_icon_name ("folder-symbolic", Gtk.IconSize.MENU);
                    icon.tooltip_text = _("Folder Manager");
                    notebook.append_page (view, icon);
                }
            });

            view.restore_settings ();
        }

        void on_hook_toolbar (Gtk.HeaderBar toolbar) {
            if (button != null) {
                return;
            }

            button = new Gtk.Button.from_icon_name ("folder-saved-search", Gtk.IconSize.LARGE_TOOLBAR);
            button.tooltip_text = _("Open a folder");
            button.clicked.connect (() => open_dialog ());
            button.show_all ();
            toolbar.pack_start (button);
        }
        
        private void open_dialog () {
            Gtk.Window window = plugins.manager.window;
            Gtk.FileChooserDialog chooser = new Gtk.FileChooserDialog (
                "Select a folder.", window, Gtk.FileChooserAction.SELECT_FOLDER,
                _("_Cancel"), Gtk.ResponseType.CANCEL,
                _("_Open"), Gtk.ResponseType.ACCEPT);
            chooser.select_multiple = true;

            if (chooser.run () == Gtk.ResponseType.ACCEPT) {
                chooser.get_files ().foreach ((file) => {
                    view.open_folder (file.get_path ());
                });
            }

            chooser.close ();
        }
    }
}

[ModuleInit]
public void peas_register_types (GLib.TypeModule module) {
  var objmodule = module as Peas.ObjectModule;
  objmodule.register_extension_type (typeof (Peas.Activatable), typeof (Scratch.Plugins.FolderManagerPlugin));
}
